package com.thuwater.data.client.push;

import java.nio.charset.StandardCharsets;
import java.util.List;

import javax.annotation.Nonnull;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.type.TypeReference;
import com.thuwater.data.client.model.ThData;
import com.thuwater.data.client.service.DataHandler;
import com.thuwater.data.client.service.TokenService;
import com.thuwater.data.client.utils.JsonUtil;

import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.codec.http.*;
import io.netty.util.AsciiString;

/**
 * @Author zhangpeng
 * @Date 2019/12/11 10:39
 */
public class HttpServerHandler extends SimpleChannelInboundHandler<HttpObject> {

    private static final Logger log = LoggerFactory.getLogger(HttpServerHandler.class);
    public static final String TOKEN = "X-TH-TOKEN";
    public static final String WHKEY = "X-TH-WHKEY";
    private static final AsciiString CONNECTION = AsciiString.cached("Connection");
    private static final AsciiString KEEP_ALIVE = AsciiString.cached("keep-alive");
    private final TokenService tokenService;
    private DataHandler dataHandler;
    private final String handleUri;

    public HttpServerHandler(@Nonnull String handleUri,
                             @Nonnull TokenService tokenService,
                             @Nonnull DataHandler dataHandler) {
        this.handleUri = handleUri;
        this.tokenService = tokenService;
        this.dataHandler = dataHandler;
    }

    @Override
    public void channelRead0(ChannelHandlerContext ctx, HttpObject msg) throws Exception {
        if (!(msg instanceof HttpRequest)) return;
        try {
            HttpRequest request = (HttpRequest) msg;
            String uri = request.uri();
            log.debug("request http uri: " + uri);
            if (!handleUri.equals(uri)) {
                // 非指定URI直接返回404
                HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.NOT_FOUND);
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);
                return;
            }
            HttpMethod method = request.method();
            if (!HttpMethod.POST.equals(method)) {
                // 非POST请求直接返回405
                HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.METHOD_NOT_ALLOWED);
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);
                return;
            }
            HttpHeaders headers = request.headers();
            String thisWhKey = headers.get(WHKEY);
            if (!tokenService.validWhKey(thisWhKey)) {
                // 如果指定了WHKEY 对于不对的key返回406
                HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.NOT_ACCEPTABLE);
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);
                return;
            }

            FullHttpRequest fullRequest = (FullHttpRequest) msg;
            String json = fullRequest.content().toString(StandardCharsets.UTF_8);
            List<ThData> datas = JsonUtil.toObject(json, new TypeReference<List<ThData>>() {});

            HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.OK);
            response.headers().set(TOKEN, tokenService.genToken());

            if (datas.isEmpty() || (datas.size() == 1 && datas.get(0).equipType.equals(0))) {
                // 收到空数据或者开关测试数据的情况 placeId=0 是测试数据
                log.info("收到空数据或开关测试数据");
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);//关闭连接
                return;
            }

            // 处理数据
            if (!dataHandler.handleDatas(datas)) {
                // 处理失败
                response.setStatus(HttpResponseStatus.SERVICE_UNAVAILABLE);
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);
                return;
            }
            // 处理成功
            boolean keepAlive = HttpUtil.isKeepAlive(request);
            if (!keepAlive) {
                ctx.write(response).addListener(ChannelFutureListener.CLOSE);
            } else {
                response.headers().set(CONNECTION, KEEP_ALIVE);
                ctx.write(response);
            }
        } catch (Exception e) {
            log.error("", e);
            HttpResponse response = new DefaultHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.INTERNAL_SERVER_ERROR);
            ctx.write(response).addListener(ChannelFutureListener.CLOSE);
            throw e;
        }
    }

    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) {
        ctx.flush();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        log.error("error", cause);
        ctx.close();
    }
}
