package com.thuwater.data.client.utils;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;


/**
 * @author zhangpeng
 */
public class JsonUtil {
    private static final Logger log = LoggerFactory.getLogger(JsonUtil.class);

    private static volatile ObjectMapper objectMapper = null;

    public static ObjectMapper objectMapperInstance() {
        if (objectMapper == null) {
            synchronized (JsonUtil.class) {
                if (objectMapper == null) {
                    objectMapper = objectMapper();
                }
            }
        }
        return objectMapper;
    }

    @SuppressWarnings("unchecked")
    private static ObjectMapper objectMapper() {
        ObjectMapper om = new ObjectMapper();
        om.setDefaultPropertyInclusion(JsonInclude.Include.NON_NULL);
        om.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
        om.setDateFormat(new SimpleDateFormat(DateTimeUtil.DEFAULT_FORMAT));
        om.enable(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES,
                  JsonParser.Feature.ALLOW_SINGLE_QUOTES,
                  JsonParser.Feature.ALLOW_UNQUOTED_CONTROL_CHARS,
                  JsonParser.Feature.ALLOW_BACKSLASH_ESCAPING_ANY_CHARACTER,
                  JsonParser.Feature.ALLOW_NUMERIC_LEADING_ZEROS,
                  JsonParser.Feature.ALLOW_TRAILING_COMMA
        );
        om.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);

        JavaTimeModule javaTimeModule = new JavaTimeModule();
        javaTimeModule.addSerializer(LocalDateTime.class, new LocalDateTimeSerializer(DateTimeUtil.DATE_TIME_FORMATTER));
        javaTimeModule.addDeserializer(LocalDateTime.class, new LocalDateTimeDeserializer(DateTimeUtil.DATE_TIME_FORMATTER));
        om.registerModule(javaTimeModule);
        return om;
    }

    @Nonnull
    public static String toJson(@Nullable Object obj) {
        if (obj == null) {
            return "{}";
        }
        ObjectMapper objectMapper = objectMapperInstance();
        try {
            return objectMapper.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            log.error("json序列化错误", e);
            return "";
        }
    }

    @Nonnull
    public static byte[] toJsonByte(@Nullable Object obj) {
        if (obj == null) {
            return new byte[0];
        }
        ObjectMapper objectMapper = objectMapperInstance();
        try {
            return objectMapper.writeValueAsBytes(obj);
        } catch (JsonProcessingException e) {
            log.error("json序列化错误", e);
            return new byte[0];
        }
    }

    @Nonnull
    public static Map<String, String> toMapFromBytes(byte[] bytes) {
        if (bytes == null) {
            return new HashMap<>();
        }
        ObjectMapper objectMapper = objectMapperInstance();
        try {
            return objectMapper.readValue(bytes, new TypeReference<Map<String, String>>() {});
        } catch (IOException e) {
            log.error("json序列化错误", e);
            return new HashMap<>();
        }
    }

    @Nonnull
    public static String toJson(@Nonnull Object obj, @Nonnull ObjectMapper objectMapper) {
        try {
            return objectMapper.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            log.error("json序列化错误", e);
            return "";
        }
    }

    /**
     * <pre>
     * returnType 示例：
     * new TypeReference&lt;List&lt;Model&gt;&gt;() {}
     * new TypeReference&lt;HashMap&lt;String, String&gt;&gt;() {}
     * </pre>
     * <br/>
     *
     * @param url
     * @param json
     * @param returnType
     * @return
     * @author "zhangpeng"
     * @since JDK 1.7
     */
    @Nonnull
    public static <T> T toObject(@Nonnull String json, @Nonnull TypeReference<T> returnType) throws Exception {
        ObjectMapper objectMapper = objectMapperInstance();
        return objectMapper.readValue(json, returnType);
    }

    @Nonnull
    public static <T> T toObject(@Nonnull String json, @Nonnull TypeReference<T> returnType, @Nonnull ObjectMapper objectMapper) throws Exception {
        return objectMapper.readValue(json, returnType);
    }

    /**
     * @author "zhangpeng"
     */
    @Nonnull
    public static Map<String, String> toHashMap(@Nonnull String json) throws Exception {
        return toObject(json, new TypeReference<Map<String, String>>() {
        });
    }

    @Nonnull
    public static Map<String, String> toHashMap(@Nonnull String json, @Nonnull ObjectMapper objectMapper) throws Exception {
        return toObject(json, new TypeReference<Map<String, String>>() {
        }, objectMapper);
    }
}
